@@ -452,6 +452,13 @@ add_addressbook_sources (EAccount *account, GSList *folders)
 		e_source_set_property (source, "offline_sync", 
 					       camel_url_get_param (url, "offline_sync") ? "1" : "0");
 		e_source_set_property (source, "completion", "true");
+		if (folder->is_default) 
+			e_source_set_property (source, "delete", "no");
+		if (folder->parent_folder_id) {
+			gchar *tmp = exchange_mapi_util_mapi_id_to_string (folder->parent_folder_id);
+			e_source_set_property (source, "parent-fid", tmp);
+			g_free (tmp);
+		}
 		e_source_group_add_source (group, source, -1);
 		g_object_unref (source);
 	}
@@ -476,6 +483,7 @@ add_addressbook_sources (EAccount *account, GSList *folders)
 		// camel_url_get_param (url, "offline_sync") ? "1" : "0");
 		e_source_set_property(source, "offline_sync", "1");
 		e_source_set_property (source, "completion", "true");
+		e_source_set_property (source, "delete", "no");
 		e_source_group_add_source (group, source, -1);
 		g_object_unref (source);
 	}
@@ -622,7 +630,7 @@ create_profile_entry (CamelURL *url)
 		g_free (key);
 
 		if (password)
-			status = exchange_mapi_create_profile (url->user, password, camel_url_get_param (url, "domain"), url->host, NULL);
+		  status = exchange_mapi_create_profile (url->user, password, camel_url_get_param (url, "domain"), url->host, NULL, NULL, NULL);
 
 		++attempts; 
 	}
@@ -678,7 +686,7 @@ mapi_account_changed (EAccountList *account_listener, EAccount *account)
 			gchar *profname = NULL, *uri = NULL; 
 			ExchangeMAPIAccountListener *config_listener = exchange_mapi_accounts_peek_config_listener();
 
-			profname = g_strdup_printf("%s@%s", new_url->user, camel_url_get_param (new_url, "domain"));
+			profname = exchange_mapi_util_profile_name (new_url->user, camel_url_get_param (new_url, "domain"));
 			camel_url_set_param(new_url, "profile", profname);
 			g_free (profname);
 
@@ -711,7 +719,7 @@ mapi_account_changed (EAccountList *account_listener, EAccount *account)
 				gchar *profname = NULL, *uri = NULL; 
 				ExchangeMAPIAccountListener *config_listener = exchange_mapi_accounts_peek_config_listener();
 
-				profname = g_strdup_printf("%s@%s", new_url->user, camel_url_get_param (new_url, "domain"));
+				profname = exchange_mapi_util_profile_name (new_url->user, camel_url_get_param (new_url, "domain"));
 				camel_url_set_param(new_url, "profile", profname);
 				g_free (profname);
 
