@@ -37,7 +37,9 @@ typedef enum {
 	MAPI_OPTIONS_FETCH_GAL = 1 <<4,
 	MAPI_OPTIONS_DONT_SUBMIT = 1<<5, 
 	MAPI_OPTIONS_GETBESTBODY = 1<<6,
-	MAPI_OPTIONS_USE_PFSTORE = 1<<7
+	MAPI_OPTIONS_USE_PFSTORE = 1<<7,
+	MAPI_OPTIONS_DONT_OPEN_MESSAGE = 1<<8,
+	MAPI_OPTIONS_DELETE_ON_SUBMIT_FAILURE = 1<<9
 } ExchangeMAPIOptions;
 
 #define MAPI_OPTIONS_FETCH_ALL MAPI_OPTIONS_FETCH_ATTACHMENTS | \
@@ -76,12 +78,9 @@ typedef struct {
 		struct SPropValue *ext_lpProps;
 	} in; 
 
-	struct {
-		/* These are properties which would be set on the 
-		 * recipients after GetRecipientTable() */
-		uint32_t all_cValues; 
-		struct SPropValue *all_lpProps;
-	} out; 
+	/* These are properties which would be set on the 
+	 * recipients after GetRecipientTable() */
+	struct SRow out_SRow;
 } ExchangeMAPIRecipient;
 
 typedef struct {
@@ -199,7 +198,9 @@ uint32_t
 exchange_mapi_util_create_named_prop (uint32_t olFolder, mapi_id_t fid, 
 				      const char *named_prop_name, uint32_t ptype);
 
-gboolean exchange_mapi_create_profile (const char *username, const char *password, const char *domain, const char *server, char **error_msg);
+gboolean exchange_mapi_create_profile (const char *username, const char *password,
+				       const char *domain, const char *server,
+				       char **error_msg, mapi_profile_callback_t cb, gpointer data);
 gboolean exchange_mapi_delete_profile (const char *profile);
 
 #endif
